﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using WMPLib;
using System.IO;

namespace Chie.m3uCome
{
  public partial class MainForm : Form
  {
    private WindowsMediaPlayerClass wmp;
    private Setting setting;

    public MainForm()
    {
      InitializeComponent();
      this.playListUpMenu.Tag = -1;
      this.playListDownMenu.Tag = 1;
      // wmp setting
      wmp = new WindowsMediaPlayerClass();
      Application.Idle += new EventHandler(idle);

      setting = new Setting(Consts.SetFile);
      copyPlace.Text = setting.CopyDirectory;
    }

    private void mainForm_FormClosed(object sender, FormClosedEventArgs e)
    {
      wmp.stop();
    }

    /// <summary>アイドルイベント。メニューのイネーブル状態切り替え</summary>
    private void idle(object sender, System.EventArgs e)
    {
      bool countNonZero = songList.Items.Count > 0; // 1曲以上登録済
      bool countNonOne = songList.Items.Count > 1; // 2曲以上登録済
      bool selected = songList.SelectedItems.Count > 0; // 選択されている
      bool selectedOne = songList.SelectedItems.Count == 1; // 選択されたアイテムは一つ
      bool selectedNonTop = selected && songList.SelectedIndices[0] > 0; // 選択は先頭項目でない
      bool selectedNonLast = selected && songList.SelectedIndices[0] < songList.Items.Count - 1; // 選択は最終項目でない

      fileCopyFiles.Enabled = countNonZero;
      fileSave.Enabled = countNonZero;
      playListFileRemoveMenu.Enabled = selected;
      playListUpMenu.Enabled = selectedOne && countNonOne && selectedNonTop;
      playListDownMenu.Enabled = selectedOne && countNonOne && selectedNonLast;
      otherTestPlay.Enabled = selectedOne;
      testPlay.Enabled = selectedOne;
      autoTestPlay.Checked = setting.AutoTestPlay;
    }
     
    #region ファイルメニュー

    /// <summary>ファイルコピー実行</summary>
    private void fileCopyFilesClick(object sender, System.EventArgs e)
    {
      try
      {
        #region 前処理
        // ディレクトリ生成処理とか
        if(!Directory.Exists(setting.CopyDirectory))
          throw new Exception("ディレクトリが存在しません。");
        // コピー先フォルダを空に
        foreach(string s in Directory.GetFiles(setting.CopyDirectory))
        {
          if(File.Exists(s))
            File.Delete(s);
        }
        #endregion
        FileCopyProsecutor fcp = new FileCopyProsecutor(setting.CopyDirectory, songList.Items);
        fcp.SavePlayList       = setting.CreateM3U ? Consts.SavePlayList : "";
        fcp.CopyOnlyMP3        = setting.MP3Only;
        //fcp.CallBack;
        /* 却下？ 2006/04/22
        if(convertMIDItoMP3)
          fcp.ConvertMIDI2MP3  = new ConvertMIDI2MP3(convertMIDItoMP3,
                                                     gogo_8hzPath,
                                                     timidityPath);
         */
        fcp.Do();        
        Chie.Dialog.Alert("終了しました");
      }catch(Exception ex){ Chie.Dialog.Error(ex.Message); }
      
    }
    /// <summary>新規メニュー。プレイリストのクリア</summary>
    private void fileNewClick(object sender, System.EventArgs e)
    {
      if(Chie.Dialog.Confirm("現在のプレイリストをすべてクリアします"))
      {
        songList.Items.Clear();
        calcFileSizes();
      }
    }

    /// <summary>リストを開く</summary>
    private void fileOpenClick(object sender, System.EventArgs e)
    {
      string fn = "";
      if(Chie.Dialog.PromptForFileName(ref fn, "", Consts.M3uFilter, Consts.M3uExt))
      {
        System.IO.StreamReader read = new System.IO.StreamReader(fn,
                                                                System.Text.Encoding.Default);
        try
        {
          this.Refresh();
          this.SuspendLayout();
          try
          {
            // ファイル読み込み
            string dir = Path.GetDirectoryName(fn);
            string line;
            songList.Items.Clear();
            while((line = read.ReadLine()) != null)
            {
              SongListItem lvi = null;
              if(File.Exists(line))
                lvi = new SongListItem(line);
              else if(File.Exists(Path.Combine(dir, line)))
                lvi = new SongListItem(Path.Combine(dir, line));
              if(lvi != null)
                songList.Items.Add(lvi);
            }
            calcFileSizes();
          }
          finally
          {
            this.ResumeLayout(true);
          }
        }
        finally
        {
          read.Close();
        }
      }
    }

    /// <summary>リストの保存</summary>
    private void fileSaveClick(object sender, System.EventArgs e)
    {
      string fn = "";
      if(Chie.Dialog.PromptForFileName(ref fn, "", Consts.M3uFilter, Consts.M3uExt, "", true))
      {
        // ファイル書き込み
        StreamWriter write = new StreamWriter(fn, false, System.Text.Encoding.Default);
        try
        {
          foreach(SongListItem si in songList.Items)
          {
            write.WriteLine(si.FileName);
          }
        }
        finally
        {
          write.Close();
        }
      }
    }

    /// <summary>終了</summary>
    private void fileExitClick(object sender, System.EventArgs e) { Application.Exit(); }
    
    #endregion

    #region プレイリストメニュー
    
    /// <summary>プレイリスト項目を追加</summary>
    private void playListFileAdd_ActionPerformed(object sender, System.EventArgs e)
    {
      string fn = "";
      if(Chie.Dialog.PromptForFileName(ref fn, "", Consts.MediaFilter, Consts.MediaExt))
        addFile(fn);
    }

    /// <summary>プレイリスト項目を削除</summary>
    private void playListFileRemove_ActionPerformed(object sender, System.EventArgs e)
    {
      if(Chie.Dialog.Confirm("削除しますか？"))
      {
        foreach(ListViewItem list in songList.SelectedItems)
          list.Remove();
        calcFileSizes();
      }
    }

    /// <summary>プレイリスト項目を上(下)へ</summary>
    private void playListMove_ActionPerformed(object sender, EventArgs e)
    {
      int prevIndex = songList.SelectedIndices[0];
      int nextIndex = prevIndex + (int)((ToolStripMenuItem)sender).Tag;
      ListViewItem item = songList.Items[prevIndex];
      songList.Items.Remove(item);
      songList.Items.Insert(nextIndex, item);
      songList.Items[nextIndex].Selected = true;
    }

    #endregion

    #region その他メニュー

    /// <summary>試聴メニュー イベントハンドラ。状態に応じて再生か停止</summary>
    private void otherTestPlayClick(object sender, System.EventArgs e)
    {
      if(wmp.playState == WMPPlayState.wmppsPlaying)
        wmp.stop();
      else
        wmp.play();
    }

    /// <summary>ディレクトリを開く イベントハンドラ</summary>
    private void otherOpenDirectoryClick(object sender, System.EventArgs e)
    {
      if(Directory.Exists(setting.CopyDirectory))
        System.Diagnostics.Process.Start(setting.CopyDirectory);
    }
    
    /// <summary>設定メニュー イベントハンドラ</summary>
    private void otherSettingClick(object sender, System.EventArgs e)
    {
      using(SettingDialog dlg = new SettingDialog())
      {
        dlg.LoadSetting(setting);
        if(dlg.ShowDialog() == DialogResult.OK)
        {
          dlg.SaveSetting(setting);
          copyPlace.Text = setting.CopyDirectory;
        }
      }
    }
    
    #endregion

    #region 追加とそれに伴う処理

    /// <summary>ファイルの実追加処理</summary>
    /// <param name="fileNames">ファイル名</param>
    private void addFile(params string[] fileNames)
    {
      foreach(string fn in fileNames)
        songList.Items.Add(new SongListItem(fn));
      calcFileSizes();
    }

    /// <summary>ファイルサイズを計算し、ステータスバーに表示します</summary>
    private void calcFileSizes()
    {
      // ファイルサイズを計算
      System.Collections.Hashtable hash = new System.Collections.Hashtable();
      long sizes = 0;
      foreach(SongListItem si in songList.Items)
      {
        string ext = Path.GetExtension(si.FileName).ToLower();
        if(hash[ext] == null)
          hash[ext] = new long();
        hash[ext] = (long)hash[ext] + si.FileSize;
        sizes += si.FileSize;
      }

      // 表示させる
      sizeList.SuspendLayout();
      try
      {
        sizeList.Items.Clear();
        foreach(DictionaryEntry data in hash)
        {
          ListViewItem item = new ListViewItem(data.Key.ToString());
          item.SubItems.Add(getFileSizeString((long)data.Value));
          sizeList.Items.Add(item);
        }

        totalFileSize.Text = string.Format("{0}({1})",
          getFileSizeString(sizes),
          songList.Items.Count.ToString());
      }
      finally
      {
        songList.ResumeLayout(true);
      }
    }

    /// <summary>バイト単位のファイルサイズを、文字列に変換します。</summary>
    /// <param name="size">バイト単位の数値</param>
    /// <returns>文字列化された数値</returns>
    private static string getFileSizeString(long size)
    {
      double fs = 0;
      string[] s = new string[] { "B", "KB", "MB", "GB", "TB" , "PB"};
      int i;
      for(i = s.Length - 1;i >= 0;i--)
      {
        // 数値表示可能な最初の桁取得
        fs = size / Math.Pow(1024, i);
        size = size % (long)Math.Pow(1024, i);
        if((int)fs != 0)
          break;
      }
      // 表示できるなら表示
      if((int)fs != 0)
        return string.Format("{0:0.#}{1}", fs, s[i]);
      else
        return "0" + s[0];
    }

    #endregion

    #region ソングリストイベントハンドラ

    private void songListSelectedIndexChanged(object sender, System.EventArgs e)
    {
      if(songList.SelectedItems.Count == 1)
      {
        wmp.stop();
        wmp.URL = ((SongListItem)songList.SelectedItems[0]).FileName;
        if(setting.AutoTestPlay)
          otherTestPlay.PerformClick();
      }
    }

    private void songListDragEnter(object sender, System.Windows.Forms.DragEventArgs e)
    {
      if(e.Data.GetDataPresent(DataFormats.FileDrop))
        e.Effect = DragDropEffects.Copy;
    }

    private void songListDragDrop(object sender, System.Windows.Forms.DragEventArgs e)
    {
      if(e.Data.GetDataPresent(DataFormats.FileDrop))
        addFile((string[])e.Data.GetData(DataFormats.FileDrop));
    }

    #endregion

    private void autoTestPlay_Click(object sender, EventArgs e)
    {
      setting.AutoTestPlay = !setting.AutoTestPlay;
    }

    /// <summary>ソングリストの各アイテムを表すクラス</summary>
    public class SongListItem : ListViewItem
    {
      private string fileName;
      private long fileSize;

      /// <summary>ファイル名を指定して、クラスを初期化します</summary>
      /// <param name="fileName">リストに表示するファイル名</param>
      public SongListItem(string fileName)
        : base()
      {
        SubItems.Add("");
        SubItems.Add("");
        this.FileName = fileName;
        using(FileStream file = new FileStream(fileName, FileMode.Open,
                                              FileAccess.Read))
          FileSize = file.Length;
      }

      /// <summary>ファイル名</summary>
      public string FileName
      {
        get { return fileName; }
        set
        {
          fileName = value;
          Text = Path.GetFileName(value);
          SubItems[2].Text = Path.GetExtension(value).ToLower();
        }
      }

      /// <summary>ファイルサイズ</summary>
      public long FileSize
      {
        get { return fileSize; }
        set
        {
          fileSize = value;
          SubItems[1].Text = MainForm.getFileSizeString(value);
        }
      }

    }

    private void fileMenu_Click(object sender, EventArgs e)
    {

    }

    private void fileMenu_Click_1(object sender, EventArgs e)
    {

    }

  }

  /// <summary>設定を保存するクラス</summary>
  /// <remarks><see cref="SettingDialog"/>で使用するために、クラスになっています(構造体ではなく)。</remarks>
  public class Setting
  {

    private string dirList;
    private string copyDirectory;
    private bool mp3Only;
    private bool createM3U;
    private bool autoTestPlay;
    /// <summary>設定をINIファイルから読み込むことで、<see cref="Setting"/>構造体を初期化します</summary>
    /// <param name="fileName">設定を読み込むINIファイル</param>
    /// <seealso cref="Chie.Ini.MemIniFile"/>
    public Setting(string fileName)
    {
      Chie.Ini.MemIniFile ini = new Chie.Ini.MemIniFile(fileName);
      dirList = ini.ReadString("DirList");
      copyDirectory = ini.ReadString("CopyDirectory");
      mp3Only = ini.ReadBool("Mp3Only");
      createM3U = ini.ReadBool("CreateM3u");
      autoTestPlay = ini.ReadBool("AutoTestPlay");

      /*  却下？ 2006/04/22
      ini.CurrentSection = "MIDI2MP3Convert";
      convertMIDItoMP3 = ini.ReadBool("Enabled");
      timidityPath = ini.ReadString("TimidityPath");
      gogo_8hzPath = ini.ReadString("Gogo_8hzPath");
       */
    }

    /// <summary>設定をINIファイルに書き込みます</summary>
    /// <param name="fileName">書き込むINIファイル</param>
    public void SaveIniFile(string fileName)
    {
      Chie.Ini.MemIniFile ini = new Chie.Ini.MemIniFile(fileName);
      // 設定の保存
      ini.WriteString("DirList", dirList);
      ini.WriteString("CopyDirectory", copyDirectory);
      ini.WriteBool("Mp3Only", mp3Only);
      ini.WriteBool("CreateM3u", createM3U);
      ini.WriteBool("AutoTestPlay", autoTestPlay);
      ini.UpdateFile();
    }

    public string[] DirList
    {
      get { return dirList.Remove(dirList.Length - 1).Split(';');      }
      set { dirList = ""; foreach(string s in value) dirList += s + ';'; }
    }

    public string CopyDirectory
    {
      get { return copyDirectory; }
      set { copyDirectory = value; }
    }

    public bool MP3Only
    {
      get { return mp3Only; }
      set { mp3Only = value; }
    }

    public bool CreateM3U
    {
      get { return createM3U; }
      set { createM3U = value; }
    }

    public bool AutoTestPlay
    {
      get { return autoTestPlay; }
      set { autoTestPlay = value; }
    }

  }

}